#include <QApplication>
#include <qtsingleapplication.h>
#include <QFileDialog>
#include <QMessageBox>
#include <QElapsedTimer>
#include <QStringList>

#include <QDebug>

#ifdef Q_OS_WIN
# include <qt_windows.h>
#else
# include <unistd.h>
#endif

int getPid()
{
#ifdef Q_OS_WIN
    return GetProcessId(GetCurrentProcess());
#else
    return getpid();
#endif
}

int main(int argc, char *argv[])
{
    const int pid = getPid();
    QElapsedTimer et;
    et.start();

    QtSingleApplication kApp( argc, argv );
    QStringList args = kApp.arguments();
    args.pop_front();

    qDebug() << "pid=" << pid << kApp.platformName() << QT_VERSION_STR << "starting" << args;

    if ( kApp.isRunning() ) {
        qDebug() << "pid=" << pid << "isRunning";
        bool result = kApp.sendMessage( "raise" );
        qDebug() << "pid=" << pid << et.elapsed() << "ms" << "kApp.sendMessage returns" << result << "terminating";
        return result ? 0 : -2;
    }

    QObject::connect(&kApp, &QtSingleApplication::messageReceived, [&pid,&et](){
        qDebug() << "pid=" << pid <<  et.elapsed() << "ms" <<"QtSingleApplication::messageReceived";
    });

    QString t = QLatin1String(QT_VERSION_STR) + QLatin1String(" pid=") + QString::number(pid);

    QFileDialog::Options opts(0);
    if (args.contains("-n")) {
        opts |= QFileDialog::DontUseNativeDialog;
        qDebug() << "pid=" << pid << opts;
    }
    QFileDialog::getOpenFileName( 0, t + " Start another instance",QString(), "Files (.)", 0, opts);
    QMessageBox::information(0, t + "Information",
                             "If the connection from another instance timed out, this application is now unresponsive.");
    qDebug() << "pid=" << pid << et.elapsed() << "ms" << "QMessageBox closed";
    return 0;
}
